#include <ansi_c.h>
#include <cvirte.h>		
#include <userint.h>
#include "p9.h"
#include "tarjeta_E_S.h"


/*****************************************************************************************/
/* Decalaracin de variables                                                             */
/*****************************************************************************************/

static int panelHandle;
int automatico=0, congela=0;
double T=0.1, ms=100, u_man=0, Kp=0, Ki=0, Kd=0, ref=0, u_auto=0, sal=0, sal_anterior=0, accion=0;
double error=0, error_anterior=0, integral_error=0, derivada_error=0;
double vector_repres[3], T_anterior, minX, maxX, offset_anterior;


/*****************************************************************************************/
/* Programa principal: carga la pantalla y ejecuta la interfaz de usuario                */
/*****************************************************************************************/

int main (int argc, char *argv[])
{
	if (InitCVIRTE (0, argv, 0) == 0)
		return -1;	/* out of memory */
	if ((panelHandle = LoadPanel (0, "p9.uir", PANEL)) < 0)
		return -1;
	init_9112 ();
	DisplayPanel (panelHandle);
	RunUserInterface ();
	DiscardPanel (panelHandle);
	analog_output_9112 (0);
	release_9112 ();
	return 0;
}


/*****************************************************************************************/
/* Timer_tick: ejecuta el algoritmo de control cada vez que se dispara el tick del timer */
/*       Juega un papel equivalente al de la rutina de servicio de interrupcin de reloj */
/*****************************************************************************************/

int CVICALLBACK Timer_tick (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_TIMER_TICK:
			sal=analog_input_9112 (0);
			error=ref-sal;
			if (automatico==1)
				{

				
				
				accion=u_auto;
				}
			else
				{
				accion=u_man;
				}
			
			analog_output_9112 (accion);
			if (congela==0)
				{
				vector_repres[0]=ref;
				vector_repres[1]=sal;
				vector_repres[2]=accion;
				PlotStripChart (panelHandle, PANEL_GRAFICA, vector_repres, 3, 0, 0,
							VAL_DOUBLE);
				}
			error_anterior=error;
			break;
		}
	return 0;
}

/****************************************************************************************/
/* Change_T: cambio del perodo de muestreo desde el elemento correspondiente del panel */
/****************************************************************************************/

int CVICALLBACK Change_T (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panelHandle, PANEL_PERIODO, &ms);
			T_anterior=T;
			T=ms/1000;
			SetCtrlAttribute (panelHandle, PANEL_TIMER, ATTR_INTERVAL, T);
			GetAxisScalingMode (panelHandle, PANEL_GRAFICA, VAL_XAXIS, 0, &minX, &maxX);
			GetCtrlAttribute (panelHandle, PANEL_GRAFICA, ATTR_XAXIS_OFFSET,&offset_anterior);
			SetCtrlAttribute (panelHandle, PANEL_GRAFICA, ATTR_POINTS_PER_SCREEN, (int)(10/T));
			SetCtrlAttribute (panelHandle, PANEL_GRAFICA, ATTR_XAXIS_GAIN, T);
			SetCtrlAttribute (panelHandle, PANEL_GRAFICA, ATTR_XAXIS_OFFSET,offset_anterior+maxX*T_anterior);
			break;
		case EVENT_RIGHT_CLICK:
			MessagePopup ("AYUDA", "Vara el perodo de muestreo. La grfica ser automticamente inicializada");
			break;
		}
	return 0;
}


/****************************************************************************************/
/* Change_accion_man: cambio de la acc. manual desde el elemento correspondiente        */
/*                    Slo tiene sentido hacerlo en modo manual							*/
/****************************************************************************************/

int CVICALLBACK Change_accion_man (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panelHandle, PANEL_ACCION_MAN, &u_man);
			break;
		case EVENT_RIGHT_CLICK:
			MessagePopup ("AYUDA", "Vara la accin manual utilizada en el modo correspondiente");
			break;
		}
	return 0;
}


/****************************************************************************************/
/* Change_Kp: cambio de la constante de ponderacin proporcional del PID                */
/****************************************************************************************/

int CVICALLBACK Change_Kp (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panelHandle, PANEL_Kp, &Kp);
			break;
		case EVENT_RIGHT_CLICK:
			MessagePopup ("AYUDA", "Vara la constante de ponderacin proporcional del PID");
			break;
		}
	return 0;
}


/****************************************************************************************/
/* Change_Ki: cambio de la constante de ponderacin integral del PID                    */
/****************************************************************************************/

int CVICALLBACK Change_Ki (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panelHandle, PANEL_Ki, &Ki);
			break;
		case EVENT_RIGHT_CLICK:
			MessagePopup ("AYUDA", "Vara la constante de ponderacin integral del PID");
			break;
		}
	return 0;
}


/****************************************************************************************/
/* Change_Kd: cambio de la constante de ponderacin derivativa del PID                  */
/****************************************************************************************/

int CVICALLBACK Change_Kd (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panelHandle, PANEL_Kd, &Kd);
			break;
		case EVENT_RIGHT_CLICK:
			MessagePopup ("AYUDA", "Vara la constante de ponderacin derivativa del PID");
			break;
		}
	return 0;
}


/****************************************************************************************/
/* Change_congelar: Activa o desactiva la representacin grfica                        */
/****************************************************************************************/

int CVICALLBACK Change_congelar (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panelHandle, PANEL_CONGELAR, &congela);
			if (congela==0) 
				{
				SetCtrlAttribute (panelHandle, PANEL_GRAFICA, ATTR_POINTS_PER_SCREEN, (int)(10/T));
				}
			break;
		case EVENT_RIGHT_CLICK:
			MessagePopup ("AYUDA", "Congela/descongela la representacin grfica");
			break;
		}
	return 0;
}


/****************************************************************************************/
/* Change_ref: cambio de la referencia desde el elemento correspondiente del panel      */
/*             Slo tiene sentido hacerlo en modo automtico  	        				*/
/****************************************************************************************/

int CVICALLBACK Change_ref (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panelHandle, PANEL_REFERENCIA, &ref);
			break;
		case EVENT_RIGHT_CLICK:
			MessagePopup ("AYUDA", "Vara la referencia utilizada en modo automtico");
			break;
		}
	return 0;
}


/****************************************************************************************/
/* Change_mode: cambio del modo de control: por defecto en modo manual (manual=true)    */
/****************************************************************************************/

int CVICALLBACK Change_mode (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			GetCtrlVal (panelHandle, PANEL_MODE, &automatico);
			if (automatico==1) 
				{
				SetCtrlVal (panelHandle, PANEL_LED_AUTO, 1);
				SetCtrlVal (panelHandle, PANEL_LED_MANUAL, 0);
				SetCtrlAttribute (panelHandle, PANEL_REFERENCIA, ATTR_DIMMED, 0);
				SetCtrlAttribute (panelHandle, PANEL_ACCION_MAN, ATTR_DIMMED, 1);
				}
			else
				{
				SetCtrlVal (panelHandle, PANEL_LED_AUTO, 0);
				SetCtrlVal (panelHandle, PANEL_LED_MANUAL, 1);
				SetCtrlAttribute (panelHandle, PANEL_REFERENCIA, ATTR_DIMMED, 1);
				SetCtrlAttribute (panelHandle, PANEL_ACCION_MAN, ATTR_DIMMED, 0);
				}
			break;
		case EVENT_RIGHT_CLICK:
			MessagePopup ("AYUDA", "Conmuta entre los modos MANUAL y AUTOMATICO.El modo mostrado en el botn se corresponde con aquel al que cambiaramos en caso de pulsar");
			break;
		}
	return 0;
}


/****************************************************************************************/
/* QuitCallback: funcin asociada al elemento "salir" del panel                         */
/*               finaliza la interfaz de usuario y con ello la aplicacin  				*/
/****************************************************************************************/

int CVICALLBACK QuitCallback (int panel, int control, int event,
		void *callbackData, int eventData1, int eventData2)
{
	switch (event)
		{
		case EVENT_COMMIT:
			QuitUserInterface (0);
			break;
		case EVENT_RIGHT_CLICK:
			MessagePopup ("AYUDA", "Salir de la aplicacin");
			break;
		}
	return 0;
}
